//
// multiply and shift
// v = (a * b) >> (s + 15)
//

module mulsh(clk, rst, a, b, s, v);
    input clk;
	 input rst;
	 input [15:0] a;
	 input [15:0] b;
	 input [3:0] s;
	 output reg [15:0] v;

  wire [17:0] fact1;
  wire [17:0] fact2;
  wire [35:0] prod;
  reg [35:0] prodBuf;
  wire [9:0] prodSign;

  assign fact1 = { a[15], a[15], a[15:0] };
  assign fact2 = { b[15], b[15], b[15:0] };

  MULT18X18 multiplier1(
    .A(fact1),    // 18-bit multiplier input
    .B(fact2),    // 18-bit multiplier input
    .P(prod)      // 36-bit multiplier output
  );

  always @(posedge clk) begin
    if (rst) begin
	   prodBuf <= 36'h000000000;
	 end else begin
      prodBuf <= prod;
	 end
  end

  assign prodSign = (prodBuf[35] == 0 ? 10'h000 : 10'hFFF);

  always @(*) begin
    case (s)
	   4'h0:  v = prodBuf[30:15];
	   4'h1:  v = prodBuf[31:16];
	   4'h2:  v = prodBuf[32:17];
	   4'h3:  v = prodBuf[33:18];
	   4'h4:  v = prodBuf[34:19];
	   4'h5:  v = prodBuf[35:20];
	   4'h6:  v = { prodSign[0:0], prodBuf[35:21] };
	   4'h7:  v = { prodSign[1:0], prodBuf[35:22] };
	   4'h8:  v = { prodSign[2:0], prodBuf[35:23] };
	   4'h9:  v = { prodSign[3:0], prodBuf[35:24] };
	   4'hA:  v = { prodSign[4:0], prodBuf[35:25] };
	   4'hB:  v = { prodSign[5:0], prodBuf[35:26] };
	   4'hC:  v = { prodSign[6:0], prodBuf[35:27] };
	   4'hD:  v = { prodSign[7:0], prodBuf[35:28] };
	   4'hE:  v = { prodSign[8:0], prodBuf[35:29] };
	   4'hF:  v = { prodSign[9:0], prodBuf[35:30] };
		default:  v = 16'hxxxx;
	 endcase
  end

endmodule
